"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useControllableState = useControllableState;
exports.useCloseOnOutsideClick = exports.useAnimatedStyles = exports.useOn = exports.useTooltip = exports.usePopover = exports.useElementByType = exports.useKeyboardDismissable = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isPlatformWeb = _reactNative.Platform.OS === 'web';
let idCounter = 0;

const generateId = prefix => {
  idCounter++;
  return 'react-native-popper-' + prefix + '-' + idCounter;
};

const useKeyboardDismissable = ({
  enabled,
  onClose
}) => {
  _react.default.useEffect(function closeOverlayOnEscapeEffectCallback() {
    let escapeKeyListener = null;

    if (_reactNative.Platform.OS === 'web') {
      escapeKeyListener = e => {
        if (e.key === 'Escape' && enabled) {
          onClose();
        }
      };

      document.addEventListener('keydown', escapeKeyListener);
    }

    return () => {
      if (_reactNative.Platform.OS === 'web') {
        document.removeEventListener('keydown', escapeKeyListener);
      }
    };
  }, [enabled, onClose]);
};

exports.useKeyboardDismissable = useKeyboardDismissable;

const useElementByType = (children, name) => {
  let element;

  _react.default.Children.forEach(children, child => {
    //@ts-ignore
    if (child.type.displayName === name) {
      element = child;
    }
  });

  return element;
};

exports.useElementByType = useElementByType;

/**
 * React hook for using controlling component state.
 * @param props
 */
function useControllableState(props) {
  const {
    value: valueProp,
    defaultValue,
    onChange
  } = props;

  const [valueState, setValue] = _react.default.useState(defaultValue);

  const isControlled = valueProp !== undefined;
  const value = isControlled ? valueProp : valueState;

  const updateValue = _react.default.useCallback(next => {
    const nextValue = typeof next === 'function' ? next(value) : next;

    if (!isControlled) {
      setValue(nextValue);
    }

    onChange && onChange(nextValue);
  }, [isControlled, onChange, value]);

  return [value, updateValue];
}

const usePopover = props => {
  const triggerId = _react.default.useRef(generateId('trigger')).current;

  const contentId = _react.default.useRef(generateId('content')).current;

  const {
    isOpen
  } = props;
  let triggerProps = {
    nativeID: triggerId,
    accessibilityRole: 'button'
  };
  let contentProps = {
    nativeID: contentId
  };
  triggerProps['aria-expanded'] = !!isOpen;
  triggerProps['aria-haspopup'] = true;
  triggerProps['aria-controls'] = isOpen ? contentId : undefined;
  contentProps.accessibilityRole = _reactNative.Platform.OS === 'web' ? 'dialog' : undefined;
  return {
    triggerProps,
    contentProps
  };
};

exports.usePopover = usePopover;

const useTooltip = props => {
  const triggerId = _react.default.useRef(generateId('trigger')).current;

  const contentId = _react.default.useRef(generateId('content')).current;

  const {
    isOpen
  } = props;
  let triggerProps = {
    nativeID: triggerId,
    accessibilityRole: 'button'
  };
  let contentProps = {
    nativeID: contentId
  };
  triggerProps['aria-describedby'] = isOpen ? contentId : undefined;
  contentProps.accessibilityRole = _reactNative.Platform.OS === 'web' ? 'tooltip' : undefined;
  return {
    triggerProps,
    contentProps
  };
};

exports.useTooltip = useTooltip;

const useOn = props => {
  let {
    on = 'press',
    onOpen
  } = props;
  let events = useHoverInteraction(props, {
    enabled: on === 'hover'
  });

  if (events) {
    return events;
  }

  if (on === 'press') {
    return {
      onPress: (0, _utils.composeEventHandlers)(props.onPress, () => {
        onOpen();
      })
    };
  }

  if (on === 'longPress') {
    return {
      onLongPress: (0, _utils.composeEventHandlers)(props.onLongPress, () => {
        onOpen();
      })
    };
  }

  return {};
};

exports.useOn = useOn;

const useHoverInteraction = (props, {
  enabled
}) => {
  // Platform won't change in between, so breaking the conditional hook rule won't be an issue

  /* eslint-disable react-hooks/rules-of-hooks */
  if (!isPlatformWeb) {
    return;
  }

  const {
    overlayRef,
    onOpen,
    onClose
  } = props;
  const {
    isHovered: isOverlayHovered
  } = useHover(overlayRef);
  const {
    isFocused: isOverlayFocused
  } = useFocus(overlayRef);

  const [isTriggerHovered, setIsTriggerHovered] = _react.default.useState(false);

  const [isTriggerFocused, setIsTriggerFocused] = _react.default.useState(false);

  _react.default.useEffect(() => {
    if (!enabled) return;

    if (isTriggerHovered || isTriggerFocused) {
      onOpen();
    }
  }, [isTriggerHovered, isTriggerFocused, onOpen, enabled]);

  _react.default.useEffect(() => {
    if (!enabled) return;
    let timeout = setTimeout(() => {
      if (!isOverlayHovered && !isTriggerHovered && !isTriggerFocused && !isOverlayFocused) {
        onClose();
      }
    });
    return () => {
      clearTimeout(timeout);
    };
  }, [isOverlayHovered, isTriggerHovered, isTriggerFocused, isOverlayFocused, onClose, enabled]);

  _react.default.useEffect(() => {
    if (!enabled) return;

    if (isTriggerFocused) {
      onOpen();
    }
  }, [isTriggerFocused, onOpen, enabled]); // We don't return above to prevent hooks conditional rules


  if (!enabled) return;
  return {
    onHoverIn: (0, _utils.composeEventHandlers)(props.onHoverIn, () => {
      setIsTriggerHovered(true);
    }),
    onHoverOut: (0, _utils.composeEventHandlers)(props.onHoverOut, () => {
      setIsTriggerHovered(false);
    }),
    onFocus: (0, _utils.composeEventHandlers)(props.onFocus, () => {
      setIsTriggerFocused(true);
    }),
    onBlur: (0, _utils.composeEventHandlers)(props.onBlur, () => {
      setIsTriggerFocused(false);
    })
  };
  /* eslint-enable react-hooks/rules-of-hooks */
};

const useHover = targetRef => {
  const [isHovered, setIsHovered] = _react.default.useState(false);

  _react.default.useEffect(() => {
    if (targetRef && targetRef.current) {
      targetRef.current.onmouseover = () => {
        setIsHovered(true);
      };

      targetRef.current.onmouseout = () => {
        setIsHovered(false);
      };
    }
  }, [targetRef]);

  return {
    isHovered
  };
};

const useFocus = targetRef => {
  const [isFocused, setIsFocused] = _react.default.useState(false);

  _react.default.useEffect(() => {
    let focusInistener = () => {
      setIsFocused(true);
    };

    let focusOutListener = () => {
      setIsFocused(false);
    };

    let currentTarget = targetRef ? targetRef.current : undefined;

    if (currentTarget) {
      currentTarget.addEventListener('focusin', focusInistener);
      currentTarget.addEventListener('focusout', focusOutListener);
    }

    return () => {
      if (currentTarget) {
        currentTarget.removeEventListener('focusin', focusInistener);
        currentTarget.removeEventListener('focusout', focusOutListener);
      }
    };
  }, [targetRef]);

  return {
    isFocused
  };
};

const useAnimatedStyles = ({
  animated,
  animationEntryDuration = 150,
  animationExitDuration = 150,
  isOpen
}) => {
  // do not animate if opened by default
  let opacity = _react.default.useRef(new _reactNative.Animated.Value(isOpen ? 1 : 0));

  const [isExited, setIsExited] = _react.default.useState(isOpen ? false : true);

  let prevIsOpen = _react.default.useRef(isOpen);

  _react.default.useEffect(() => {
    // Opened
    if (!prevIsOpen.current && isOpen) {
      _reactNative.Animated.timing(opacity.current, {
        toValue: 1,
        duration: animationEntryDuration,
        useNativeDriver: true
      }).start(() => setIsExited(false));
    } // Closed
    else if (prevIsOpen.current && !isOpen) {
        _reactNative.Animated.timing(opacity.current, {
          toValue: 0,
          duration: animationExitDuration,
          useNativeDriver: true
        }).start(() => setIsExited(true));
      }

    prevIsOpen.current = isOpen;
  }, [isOpen, animationEntryDuration, animationExitDuration]);

  if (!animated) {
    return {
      styles: {},
      isExited
    };
  }

  return {
    styles: {
      opacity: opacity.current
    },
    isExited
  };
};

exports.useAnimatedStyles = useAnimatedStyles;

const useCloseOnOutsideClick = ({
  refs,
  onClose,
  enabled
}) => {
  _react.default.useEffect(() => {
    if (!enabled) return;

    let listener = e => {
      let clickedInRefs = false;

      if (refs) {
        refs.forEach(ref => {
          if (ref && ref.current && ref.current.contains(e.target)) {
            clickedInRefs = true;
          }
        });
      }

      if (!clickedInRefs) onClose();
    };

    if (isPlatformWeb) {
      document.addEventListener('click', listener);
    }

    return () => {
      if (isPlatformWeb) {
        document.removeEventListener('click', listener);
      }
    };
  }, [refs, onClose, enabled]);
};

exports.useCloseOnOutsideClick = useCloseOnOutsideClick;
//# sourceMappingURL=index.js.map