import React from 'react';
import { Animated, Platform } from 'react-native';
import { composeEventHandlers } from '../utils';
const isPlatformWeb = Platform.OS === 'web';
let idCounter = 0;

const generateId = prefix => {
  idCounter++;
  return 'react-native-popper-' + prefix + '-' + idCounter;
};

export const useKeyboardDismissable = ({
  enabled,
  onClose
}) => {
  React.useEffect(function closeOverlayOnEscapeEffectCallback() {
    let escapeKeyListener = null;

    if (Platform.OS === 'web') {
      escapeKeyListener = e => {
        if (e.key === 'Escape' && enabled) {
          onClose();
        }
      };

      document.addEventListener('keydown', escapeKeyListener);
    }

    return () => {
      if (Platform.OS === 'web') {
        document.removeEventListener('keydown', escapeKeyListener);
      }
    };
  }, [enabled, onClose]);
};
export const useElementByType = (children, name) => {
  let element;
  React.Children.forEach(children, child => {
    //@ts-ignore
    if (child.type.displayName === name) {
      element = child;
    }
  });
  return element;
};

/**
 * React hook for using controlling component state.
 * @param props
 */
export function useControllableState(props) {
  const {
    value: valueProp,
    defaultValue,
    onChange
  } = props;
  const [valueState, setValue] = React.useState(defaultValue);
  const isControlled = valueProp !== undefined;
  const value = isControlled ? valueProp : valueState;
  const updateValue = React.useCallback(next => {
    const nextValue = typeof next === 'function' ? next(value) : next;

    if (!isControlled) {
      setValue(nextValue);
    }

    onChange && onChange(nextValue);
  }, [isControlled, onChange, value]);
  return [value, updateValue];
}
export const usePopover = props => {
  const triggerId = React.useRef(generateId('trigger')).current;
  const contentId = React.useRef(generateId('content')).current;
  const {
    isOpen
  } = props;
  let triggerProps = {
    nativeID: triggerId,
    accessibilityRole: 'button'
  };
  let contentProps = {
    nativeID: contentId
  };
  triggerProps['aria-expanded'] = !!isOpen;
  triggerProps['aria-haspopup'] = true;
  triggerProps['aria-controls'] = isOpen ? contentId : undefined;
  contentProps.accessibilityRole = Platform.OS === 'web' ? 'dialog' : undefined;
  return {
    triggerProps,
    contentProps
  };
};
export const useTooltip = props => {
  const triggerId = React.useRef(generateId('trigger')).current;
  const contentId = React.useRef(generateId('content')).current;
  const {
    isOpen
  } = props;
  let triggerProps = {
    nativeID: triggerId,
    accessibilityRole: 'button'
  };
  let contentProps = {
    nativeID: contentId
  };
  triggerProps['aria-describedby'] = isOpen ? contentId : undefined;
  contentProps.accessibilityRole = Platform.OS === 'web' ? 'tooltip' : undefined;
  return {
    triggerProps,
    contentProps
  };
};
export const useOn = props => {
  let {
    on = 'press',
    onOpen
  } = props;
  let events = useHoverInteraction(props, {
    enabled: on === 'hover'
  });

  if (events) {
    return events;
  }

  if (on === 'press') {
    return {
      onPress: composeEventHandlers(props.onPress, () => {
        onOpen();
      })
    };
  }

  if (on === 'longPress') {
    return {
      onLongPress: composeEventHandlers(props.onLongPress, () => {
        onOpen();
      })
    };
  }

  return {};
};

const useHoverInteraction = (props, {
  enabled
}) => {
  // Platform won't change in between, so breaking the conditional hook rule won't be an issue

  /* eslint-disable react-hooks/rules-of-hooks */
  if (!isPlatformWeb) {
    return;
  }

  const {
    overlayRef,
    onOpen,
    onClose
  } = props;
  const {
    isHovered: isOverlayHovered
  } = useHover(overlayRef);
  const {
    isFocused: isOverlayFocused
  } = useFocus(overlayRef);
  const [isTriggerHovered, setIsTriggerHovered] = React.useState(false);
  const [isTriggerFocused, setIsTriggerFocused] = React.useState(false);
  React.useEffect(() => {
    if (!enabled) return;

    if (isTriggerHovered || isTriggerFocused) {
      onOpen();
    }
  }, [isTriggerHovered, isTriggerFocused, onOpen, enabled]);
  React.useEffect(() => {
    if (!enabled) return;
    let timeout = setTimeout(() => {
      if (!isOverlayHovered && !isTriggerHovered && !isTriggerFocused && !isOverlayFocused) {
        onClose();
      }
    });
    return () => {
      clearTimeout(timeout);
    };
  }, [isOverlayHovered, isTriggerHovered, isTriggerFocused, isOverlayFocused, onClose, enabled]);
  React.useEffect(() => {
    if (!enabled) return;

    if (isTriggerFocused) {
      onOpen();
    }
  }, [isTriggerFocused, onOpen, enabled]); // We don't return above to prevent hooks conditional rules

  if (!enabled) return;
  return {
    onHoverIn: composeEventHandlers(props.onHoverIn, () => {
      setIsTriggerHovered(true);
    }),
    onHoverOut: composeEventHandlers(props.onHoverOut, () => {
      setIsTriggerHovered(false);
    }),
    onFocus: composeEventHandlers(props.onFocus, () => {
      setIsTriggerFocused(true);
    }),
    onBlur: composeEventHandlers(props.onBlur, () => {
      setIsTriggerFocused(false);
    })
  };
  /* eslint-enable react-hooks/rules-of-hooks */
};

const useHover = targetRef => {
  const [isHovered, setIsHovered] = React.useState(false);
  React.useEffect(() => {
    if (targetRef && targetRef.current) {
      targetRef.current.onmouseover = () => {
        setIsHovered(true);
      };

      targetRef.current.onmouseout = () => {
        setIsHovered(false);
      };
    }
  }, [targetRef]);
  return {
    isHovered
  };
};

const useFocus = targetRef => {
  const [isFocused, setIsFocused] = React.useState(false);
  React.useEffect(() => {
    let focusInistener = () => {
      setIsFocused(true);
    };

    let focusOutListener = () => {
      setIsFocused(false);
    };

    let currentTarget = targetRef ? targetRef.current : undefined;

    if (currentTarget) {
      currentTarget.addEventListener('focusin', focusInistener);
      currentTarget.addEventListener('focusout', focusOutListener);
    }

    return () => {
      if (currentTarget) {
        currentTarget.removeEventListener('focusin', focusInistener);
        currentTarget.removeEventListener('focusout', focusOutListener);
      }
    };
  }, [targetRef]);
  return {
    isFocused
  };
};

export const useAnimatedStyles = ({
  animated,
  animationEntryDuration = 150,
  animationExitDuration = 150,
  isOpen
}) => {
  // do not animate if opened by default
  let opacity = React.useRef(new Animated.Value(isOpen ? 1 : 0));
  const [isExited, setIsExited] = React.useState(isOpen ? false : true);
  let prevIsOpen = React.useRef(isOpen);
  React.useEffect(() => {
    // Opened
    if (!prevIsOpen.current && isOpen) {
      Animated.timing(opacity.current, {
        toValue: 1,
        duration: animationEntryDuration,
        useNativeDriver: true
      }).start(() => setIsExited(false));
    } // Closed
    else if (prevIsOpen.current && !isOpen) {
        Animated.timing(opacity.current, {
          toValue: 0,
          duration: animationExitDuration,
          useNativeDriver: true
        }).start(() => setIsExited(true));
      }

    prevIsOpen.current = isOpen;
  }, [isOpen, animationEntryDuration, animationExitDuration]);

  if (!animated) {
    return {
      styles: {},
      isExited
    };
  }

  return {
    styles: {
      opacity: opacity.current
    },
    isExited
  };
};
export const useCloseOnOutsideClick = ({
  refs,
  onClose,
  enabled
}) => {
  React.useEffect(() => {
    if (!enabled) return;

    let listener = e => {
      let clickedInRefs = false;

      if (refs) {
        refs.forEach(ref => {
          if (ref && ref.current && ref.current.contains(e.target)) {
            clickedInRefs = true;
          }
        });
      }

      if (!clickedInRefs) onClose();
    };

    if (isPlatformWeb) {
      document.addEventListener('click', listener);
    }

    return () => {
      if (isPlatformWeb) {
        document.removeEventListener('click', listener);
      }
    };
  }, [refs, onClose, enabled]);
};
//# sourceMappingURL=index.js.map